//******************************************************************************
//*
//* USB DAC 32 - Hi-resolution system
//*
//* AUDIODESIGNGUIDE 
//*
//* all right reserved - @copyright
//*
//******************************************************************************
#include  <htc.h>
#include  <string.h>
#include  <stdio.h>
#include  <stdlib.h>
#include  "877_i2c.h"


#define PIC_CLK 20000000
#include "delay_alternative_enhanced_precision.h"


__CONFIG(FOSC_HS & PWRTE_ON & WDTE_OFF & DEBUG_OFF & LVP_OFF & WDTE_OFF & BOREN_ON & CP_OFF);

#define byte unsigned char

//******************************************************************************
//* function declarationis
//******************************************************************************
void            LCD_init(void);
void            LCD_write_str(int, int, char *);
void            LCD_write_data(unsigned char);
void            LCD_write_cmd(unsigned char);
void            LCD_wait_busy(void);
void            LCD_clear_screen();
void            LCD_cursor_on();
byte            LCD_get_byte(void);
void            Check_sampler();
void            SetVolume(void);
void            SetMute(void);
void            SetInput(void);
void            Check_state();
void            DelayMs(int);
unsigned char   IR_readByte(int inv, int CheckTime);
unsigned char * itoa2(char * buf, int i, unsigned char max_char);
void write_ext_eeprom(unsigned char address, unsigned char data);

//******************************************************************************
//* define
//******************************************************************************
#define PORTBIT(adr, bit) ((unsigned)(&adr)*8+(bit))

static bit LCD_RS     @ PORTBIT(PORTC, 0);
static bit LCD_RW     @ PORTBIT(PORTC, 6);
static bit LCD_EN     @ PORTBIT(PORTC, 7);
static bit LCD_O1     @ PORTBIT(TRISC, 0);
static bit LCD_O2     @ PORTBIT(TRISC, 6);
static bit LCD_O3     @ PORTBIT(TRISC, 7);

#define    LCD_DATA   PORTD
#define    LCD_TRIS   TRISD

static bit LCD_D7     @ PORTBIT(PORTD, 7);
static bit LCD_D6     @ PORTBIT(PORTD, 6);
static bit LCD_D5     @ PORTBIT(PORTD, 5);
static bit LCD_D4     @ PORTBIT(PORTD, 4);
static bit LCD_T7     @ PORTBIT(TRISD, 7);
static bit LCD_T6     @ PORTBIT(TRISD, 6);
static bit LCD_T5     @ PORTBIT(TRISD, 5);
static bit LCD_T4     @ PORTBIT(TRISD, 4);

static bit LCD_BKL    @ PORTBIT(PORTD, 3);
static bit RELAY      @ PORTBIT(PORTD, 2);

// Only for XMOS interface
static bit SAM_44     @ PORTBIT(PORTA, 5);
static bit SAM_88     @ PORTBIT(PORTA, 4);
static bit SAM_176    @ PORTBIT(PORTA, 3);
static bit USB_state  @ PORTBIT(PORTA, 2);
static bit RX_state   @ PORTBIT(PORTA, 1);
static bit SAM_192    @ PORTBIT(PORTA, 0);
static bit SAM_96     @ PORTBIT(PORTE, 2);
static bit SAM_48     @ PORTBIT(PORTE, 1);
static bit AUXOUT     @ PORTBIT(PORTE, 0);

// Only if keyboard is used
static bit BUT_07     @ PORTBIT(PORTB, 7);
static bit BUT_06     @ PORTBIT(PORTB, 6);
static bit BUT_05     @ PORTBIT(PORTB, 5);
static bit BUT_04     @ PORTBIT(PORTB, 4);
static bit BUT_03     @ PORTBIT(PORTB, 3);
static bit BUT_02     @ PORTBIT(PORTB, 2);
static bit BUT_01     @ PORTBIT(PORTB, 1);
static bit IR         @ PORTBIT(PORTB, 0);

// I2C pins
static bit I2C_SCL    @ PORTBIT(PORTC, 3);
static bit I2C_SDA    @ PORTBIT(PORTC, 4);

// Not used
static bit I2C_RST    @ PORTBIT(PORTC, 1);
static bit I2C_MUT    @ PORTBIT(PORTC, 2);
static bit I2C_SDO    @ PORTBIT(PORTC, 5);
static bit I2C_LE1    @ PORTBIT(PORTD, 0);
static bit I2C_LE2    @ PORTBIT(PORTD, 1);

// Infrared codes
#define    IR_ON     (0x1C)
#define    IR_UP     (0x10)
#define    IR_DOWN   (0x1D)
#define    IR_SET    (0x11)
#define    IR_FWR    (0x18)
#define    IR_REW    (0x15)
#define    IR_SETOFF (0x19)

//******************************************************************************
//* global variables
//******************************************************************************
unsigned char   tmp[17], tmp2[8];
int             level   = 60;
unsigned char   IR_data, IR_addr, save_addr, save_USB_state = 2, save_RX_state = 2;
int             key = 0;
int             sampler = 44;
char            def = 0;
char            lcd = 0;
char            mute = 0;
char            input = 0;
unsigned char   first = 1;

//******************************************************************************
//* constant variables
//******************************************************************************
char Usb_link_ok[]     = "USB ok";
char Usb_link_ko[]     = "USB ko";
char Audio_stream_ok[] = "RX data";
char Audio_stream_ko[] = "No data";
char Banner1[]         = "AUDIODESIGNGUIDE";
char Banner2[]         = "    QUANGHA0    ";
char Volume_set[]      = "VOL    %";
char KHz[]             = "KHz";
char InI2S[]           = "I2S/DSD";
char InSpdif1[]        = "Spdif 1";
char InSpdif2[]        = "Spdif 2";
char InSpdif3[]        = "Spdif 3";



main()
{
    int i;

    //nPOR = 0;
    //nBOR = 0;
    //nPOR = 1;
    //nBOR = 1;

    PORTA   = 0b00000000;  // Clear PORTA
    PORTB   = 0b00000000;  // Clear PORTB
    PORTC   = 0b00000000;  // Clear PORTC
    PORTD   = 0b00000000;  // Clear PORTD
    PORTE   = 0b00000000;  // Clear PORTE

    ADCON1  = 0b00001110;  // All posts set to digital I/O
    ADCON1  = 0x6;         // All posts set to digital I/O

    TRISA   = 0b00111111;  // All ports A are input
    TRISB   = 0b11111111;  // All ports B are input
    TRISC   = 0b00000000;  // All ports C are output
    TRISD   = 0b00000000;  // All ports D are output
    TRISE   = 0b00000110;  // RE1 and RE2 ports are input

    nRBPU   = 0;           // PORTB pull-ups are enabled by individual port latch values

    //***********************************************************
    //* I2C & DAC startup
    //***********************************************************
    i2c_init();    // init i2c 

    // Register #17
    // default 0b00011100
    //                  +--- 0: normal 8 ch mode   1: mono
    //                 +---- 0: phase normal
    //                +----- 1: 2nd stage FIR 28   0: 27
    //               +------ 1: auto detect spdif
    //              +------- 1: Deemphasis enable
    //             +-------- 0: disable force reclock
    //            +--------- 0: disable force dll_lock
    //           +---------- 0: left if mono
    // new set 0b00000100
    //                  +--- 0: normal 8 ch mode   1: mono
    //                 +---- 0: phase normal
    //                +----- 1: 2nd stage FIR 28   0: 27
    //               +------ 0: disable auto detect spdif
    //              +------- 0: Deemphasis disabled 
    //             +-------- 0: disable force reclock
    //            +--------- 0: disable force dll_lock
    //           +---------- 0: left if mono
    //
    // disable auto spdif
    write_ext_eeprom(17,0b00010100);
        //write_ext_eeprom(17,0x4);   // 100

    // set initial input to I2S
    SetInput();

    // set initial volume
    SetVolume();


    //***********************************************************
    //* display startup
    //***********************************************************
    LCD_BKL = 1;

    LCD_init();

    DelayMs(50);

    //***********************************************************
    //* display banner
    //***********************************************************
    LCD_write_str( 1, 1, Banner1);
    LCD_write_str( 2, 1, Banner2);

    DelayMs(2000);

    //***********************************************************
    //* clear screen 
    //***********************************************************
    LCD_write_cmd(0b00000001);
    DelayMs(5);

    LCD_write_str( 1, 9, Volume_set);

    //LCD_write_str( 1, 4, KHz);

    lcd = 1;

    // recall these functions to write on LCD
    SetInput();
    SetVolume();

    //***********************************************************
    //* starup interrupts
    //***********************************************************
    INTEDG = 0;
    INTE = 1;
    GIE = 1;

    //***********************************************************
    //* main loop
    //***********************************************************
    while(1)
    {
	key = 0;

        //***********************************************************************
        //* get input from infrared receiver
        //***********************************************************************
	if (IR_addr == 0x20)
	{
	    IR_addr = 0;

	    key = IR_data;
	}
        //***********************************************************************
        //* get input from button
        //***********************************************************************
	if (!BUT_03) key = IR_UP;
	if (!BUT_05) key = IR_DOWN;
	if (!BUT_04) key = IR_ON;
	if (!BUT_01) key = IR_SET;

        //***********************************************************************
        //* stop interrupts until the work is finished
        //***********************************************************************
	if (key != 0)
	{
	    GIE = 0;

	    first = 0;
	}

        //***********************************************************************
        //* elaborate input
        //***********************************************************************
	 switch(key)
	 {
	    case IR_FWR:
		if (input < 3)
		    input++;

		SetInput();
		break;

            case IR_REW:
		if (input > 0)
		    input--;

		SetInput();
		break;

	    case IR_ON:
		SetMute();
		break;

	    case IR_DOWN:
		if (level < 99)
			level++;

		SetVolume();
		break;

	    case IR_UP:
		if (level > 0)
			level--;

		SetVolume();
		break;

	    case IR_SET:
		strcpy(tmp, "SET ");

		LCD_write_str( 1, 13, tmp);

		write_ext_eeprom(17,0x4);   // 100
		break;
	}

        //***********************************************************************
        //* start interrupts
        //***********************************************************************
	if (key != 0 ) GIE = 1;

        //***********************************************************
        //* set startup volume to zero until no command is received
        //***********************************************************
	//if (first)
	//{
	    //SetVolume(0);
	//}

	DelayMs(100);
    }
}


void write_ext_eeprom(unsigned char address, unsigned char data)
 {
   i2c_start();
   i2c_write(0x90);
   i2c_write(address);
   i2c_write(data);
   i2c_stop();
   DelayMs(11);
}

void SetMute()
{
    if (mute)
    {
	write_ext_eeprom(10,0b11001110);

	LCD_write_str( 1, 13, itoa2(tmp, 100 - level, 3));

	strcpy(tmp, "%");

	LCD_write_str( 1, 16, tmp);

        mute = 0;
    }
    else
    {
	write_ext_eeprom(10,0b11001111);

	strcpy(tmp, "MUTE");

	LCD_write_str( 1, 13, tmp);

        mute = 1;
    }

    return;
}

void SetVolume()
{
    if (mute)  // unmute
    {
        SetMute();
        return;
    }

    write_ext_eeprom(0,level);
    write_ext_eeprom(1,level);
    write_ext_eeprom(2,level);
    write_ext_eeprom(3,level);
    write_ext_eeprom(4,level);
    write_ext_eeprom(5,level);
    write_ext_eeprom(6,level);
    write_ext_eeprom(7,level);

    if (lcd)
    {
         LCD_write_str( 1, 13, itoa2(tmp, 100 - level, 3));

         strcpy(tmp, "%");

         LCD_write_str( 1, 16, tmp);
    }
    return;
}

void SetInput()
{
    SetVolume();

    switch (input)
    {
	    case 0:
		// disable auto spdif
		write_ext_eeprom(17,0b00010100);

		// disable spdif and enable I2S
		write_ext_eeprom(8,0b00000000);

		strcpy(tmp, InI2S);
		break;
	    case 1:
		// enable auto spdif
		write_ext_eeprom(17,0b00011100);

		// disable I2S and enable spdif
		write_ext_eeprom(8,0b10000000);

		write_ext_eeprom(18,32);   

		strcpy(tmp, InSpdif1);
		break;
	    case 2:
		// enable auto spdif
		write_ext_eeprom(17,0b00011100);

		// disable I2S and enable spdif
		write_ext_eeprom(8,0b10000000);

		write_ext_eeprom(18,64);   

		strcpy(tmp, InSpdif2);
		break;
	    case 3:
		// enable auto spdif
		write_ext_eeprom(17,0b00011100);

		// disable I2S and enable spdif
		write_ext_eeprom(8,0b10000000);

		write_ext_eeprom(18,128);   

		strcpy(tmp, InSpdif3);
		break;
    }
    if (lcd)
        LCD_write_str( 2, 1, tmp);

    return;
}



void Check_state()
{
    //***********************************************************************
    //* display USB state 
    //***********************************************************************
    if (USB_state != save_USB_state)
    {
	if (USB_state == 1)
	    LCD_write_str( 2, 1, Usb_link_ok);
	else
	    LCD_write_str( 2, 1, Usb_link_ko);
			
	save_USB_state = USB_state;
    }
    //***********************************************************************
    //* display RX data state
    //***********************************************************************
    if (RX_state != save_RX_state)
    {
    	if (RX_state == 1)
	    LCD_write_str( 2, 10, Audio_stream_ok);
	else
	    LCD_write_str( 2, 10, Audio_stream_ko);

    	save_RX_state = RX_state;
    }
    return;
}

void Check_sampler()
{
    unsigned char byte1, byte2, byte3, byte4;
    int test = 0;

    if      ( SAM_44  ) test = 44;
    else if ( SAM_48  ) test = 48;
    else if ( SAM_88  ) test = 88;
    else if ( SAM_96  ) test = 96;
    else if ( SAM_176 ) test = 176;
    else if ( SAM_192 ) test = 192;
    else                test = 192;

    if ( sampler == test )
	 return;

    sampler = test;

    LCD_write_str( 1, 1, itoa2(tmp, sampler, 3));

    return;
}


//***********************************************************************
//* infrared receive
//***********************************************************************
static void interrupt isr(void)
{
#define MAX_NEG_START  (10000)
#define MIN_NEG_START  (8000)
#define MAX_POS_START  (5500)
#define MIN_POS_START  (3500)
#define MAX_POS_REP    (3000)
#define MIN_POS_REP    (1000)
#define MIN_POS_BIT1   (1000)
#define DELAY_STEP     (250)

    int   l, k;
    unsigned char  data1, data2, addr1, addr2;




    if (INTF == 0) return;

    INTF = 0;


    if (IR) return;

    
    for (k = 0; k < MAX_NEG_START && !IR; k += DELAY_STEP)
    {
	DelayUs(DELAY_STEP);
    }


    if (k >= MAX_NEG_START) return;
    if (k <= MIN_NEG_START) return;

    for (l = 0; l < MAX_POS_START && IR; l += DELAY_STEP)
    {
	DelayUs(DELAY_STEP);
    }

    if (l < MAX_POS_REP && l > MIN_POS_REP)
    {
    	while(!IR);

	IR_addr = save_addr;

	return;
    }

    if (l >= MAX_POS_START) return;
    if (l <= MIN_POS_START) return;

    while(!IR);

    addr1 = IR_readByte(0, MIN_POS_BIT1);

    addr2 = IR_readByte(1, MIN_POS_BIT1);

    data1 = IR_readByte(0, MIN_POS_BIT1);

    data2 = IR_readByte(1, MIN_POS_BIT1);



    if (addr1 != addr2) return;
    if (data1 != data2) return;


    if (data1 == 0)  return;

    save_addr = addr1;

    IR_addr = addr1;
    IR_data = data1;


    return;
}


unsigned char IR_readByte(int inv, int CheckTime)
{
    unsigned char out;
    int i, k;

    for (k = 0; k < 8; k++)
    {
	for (i = 0; IR; i += DELAY_STEP)
	{
		DelayUs(DELAY_STEP);
	}
        while (!IR);

        if (i > CheckTime)
        {
	    if (inv == 0 ) 
		tmp2[k] = '1';
	    else
                tmp2[k] = '0';
        }
        else
        {
	    if (inv == 0 ) 
		tmp2[k] = '0';
	    else
                tmp2[k] = '1';

        }
    }

    out = 0;

    for (k = 7; k >= 0; k--)
    {
	if (tmp2[k] == '1')
	{
	    out <<= 1; out |= 1;
	}
	else
	{
	    out <<= 1;
	}
    }
    return out;
}


//*********************************************************************
//* LCD init, read and write
//*********************************************************************
void LCD_init()
{
    LCD_O1 = 0;
    LCD_O2 = 0;
    LCD_O3 = 0;

    LCD_T7 = 0;
    LCD_T6 = 0;
    LCD_T5 = 0;
    LCD_T4 = 0;

    LCD_RW = 0;
    LCD_RS = 0;		

    //********* 2 lines & display on ******************
    LCD_write_cmd(0b00101100);
    DelayUs(100);

    //********* display on & cursor off ******************
    LCD_write_cmd(0b00001100);
    DelayUs(100);

    //********* clear screen ******************
    LCD_write_cmd(0b00000001);
    DelayMs(5);

    //********* incr. mode ******************
    LCD_write_cmd(0b00000110);
    DelayUs(100);
}

byte LCD_get_byte(void)
{
#define	LCD_TRIS_DATAMASK	((byte)(0b11110000))
#define	LCD_D4_BIT		4
    byte  retval, highbits;


    LCD_T7 = 1;
    LCD_T6 = 1;
    LCD_T5 = 1;
    LCD_T4 = 1;

    LCD_RS = 0;		
    LCD_RW = 1;		// read


    LCD_EN = 1;
    highbits = (((LCD_DATA & LCD_TRIS_DATAMASK) >> LCD_D4_BIT) << 4);
    LCD_EN = 0;

    DelayUs(1);

    LCD_EN = 1;
    retval = ((LCD_DATA & LCD_TRIS_DATAMASK) >> LCD_D4_BIT);
    LCD_EN = 0;

    retval |= highbits;

    LCD_T7 = 0;
    LCD_T6 = 0;
    LCD_T5 = 0;
    LCD_T4 = 0;

    return retval;
}


void LCD_put_byte(byte c)
{
    c = c << LCD_D4_BIT;

    c &= LCD_TRIS_DATAMASK;

    LCD_DATA = (LCD_DATA & ~LCD_TRIS_DATAMASK) | c;

    DelayUs(1);

    LCD_EN = 1;
    DelayUs(2);
    LCD_EN = 0;
}


void LCD_write_cmd(unsigned char c)
{
    while (LCD_get_byte() & 0x80);

    LCD_RW = 0;
    LCD_RS = 0;		

    LCD_put_byte(c >> 4);
    LCD_put_byte(c);
}

void LCD_write_data(unsigned char c)
{
    while (LCD_get_byte() & 0x80);

    LCD_RW = 0;
    LCD_RS = 1;		

    LCD_put_byte(c >> 4);
    LCD_put_byte(c);
}

void LCD_write_str(int line, int pos, unsigned char * str)
{
    unsigned char    curr;
    unsigned char    i;


    //*************************************************
    // set the posistion and line
    //*************************************************
    if (line == 1)
	curr = 0b10000000 + (pos - 1);
    else
	curr = 0b11000000 + (pos - 1);

    LCD_write_cmd(curr);

    DelayUs(50);

    //*************************************************
    // send the string
    //*************************************************
    for(i = 0; str[i] != '\0'; i++)
    {
        LCD_write_data(str[i]);
    }
}


//***************************************************************************
// common functions
//***************************************************************************
void DelayMs(int cnt)
{
    unsigned char	i;

    do {
	i = 4;

	do {
		DelayUs(250);
	    } while(--i);
    } while(--cnt);
}


unsigned char * itoa2(char * buf, int i, unsigned char max_char)
{
    char *p = buf + max_char;

    unsigned char k = 0;

    *p = '\0';

    do {
          *--p = '0' + (i % 10);

          i /= 10;

          k++;

    } while (i != 0);

    for ( ; k < max_char; k++)
        *--p = ' ';

    return p;
}
